# Generated by Django 3.0.4 on 2020-04-02 09:15

from django.db import migrations, models

from image_assets import defaults


class Migration(migrations.Migration):

    dependencies = [
        ('image_assets', '0002_models_localization'),
    ]

    operations = []

    if defaults.ASSET_MODEL == 'image_assets.Asset':
        operations.extend([
            migrations.AlterUniqueTogether(
                name='deletedasset',
                unique_together=set(),
            ),
        ])

    if defaults.DELETED_ASSET_MODEL == 'image_assets.DeletedAsset':
        operations.extend([
            migrations.AddConstraint(
                model_name='asset',
                constraint=models.UniqueConstraint(
                    condition=models.Q(active=True),
                    fields=('asset_type', 'content_type', 'object_id'),
                    name='unique_active_asset'),
            ),
        ])
